# comic-get.pl - Utility for fetching comics from the internet
# Copyright (C) 2004-2007 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package ComicGet::Fetch;

use strict;
use warnings FATAL => 'all';

use IO::Socket;

use ComicGet::Config;
use ComicGet::Web;


my $file_no = 0;
my $act_line;


=item list_to_hash
@: List
r: Hash
=cut
sub list_to_hash(@) {
	my %hash;
	foreach my $line (@_) {
		my (@t) = split(/\t/, $line);
		$hash{$t[0]} = $t[1];
	}
	return %hash;
}


sub uniq_push(\@@) {
	my $dest_ref = shift;
	my @candidates = @_;

	foreach my $cand (@candidates) {
		my $conflict = 0;
		foreach my $old (@{$dest_ref}) {
			$conflict = 1 if ($old eq $cand);
		}
		push(@{$dest_ref}, $cand) if ($conflict == 0);
	}
}


=item get
0: Page
1: Type
@: Params
r: Target
=cut
sub get($$@) {
	my $page = shift;
	my $type = shift;
	my $mparam = shift;
	my @ret;
	my ($uri, $t0, $t1);
	my ($otag, $stag, $etag, $opt);
	$mparam =~ s/ /.*?/;

	if ($type eq "image") {
		$otag = "<img";
		$etag = ">";
		$opt = "src";
	}
	elsif ($type eq "link") {
		$otag = "<a";
		$etag = "</a>";
		$opt = "href";
	}
	elsif ($type eq "frame") {
		$otag = "<i?frame";
		$etag = "</i?frame>";
		$opt = "src";
	} else {
		die("Unknown type for get: $type");
	}

	# Assume no '<'s in URIs
	$ENV{'DEBUG'} && print "search: $otag(\\s[^<>]*?$mparam.*?)$etag\n";
	foreach ($page =~ /$otag(\s[^<>]*?$mparam.*?)$etag/isg) {
		$ENV{'DEBUG'} && print "pre-match: \"$_\"\n";
		# look for all rules
		my $match = ($#_ == -1) ? 1 : 0;
		foreach my $rule (@_) {
			if (/$rule/is) {
				$match = 1;
				last;
			}
		}
		next unless ($match == 1);

		($_) = /\s$opt\s*=\s*(.*)/is;
		s/\n//g;
		($t0, $t1) = /(.)(.*)/;
		# Was it 'foo="uri"', 'foo="uri"' or "foo=uri"?
		if ($t0 =~ /['"]/) {
			($uri) = $t1 =~ /(.*?)$t0/;
		} else {
			$t1 = $t0 . $t1;
			($uri) = $t1 =~ /(.*?)\s/;
		}

		next unless (defined($uri));
		&uniq_push(\@ret, &ComicGet::Web::complete_uri($uri));
	}

	return @ret;
}


=item get_meta
0: Page
1: Tag
2: Match
@: Params
r: Target
=cut
sub get_tag($@) {
	my $page = shift;
	my $tag = shift;
	my $match = shift;
	my $params;
	my @ret;

	foreach my $xx ($page =~ /<$tag\s([^>]*.*?)>/igs) {
		next unless ($xx =~ /$match/);
		my $ok = ($#_ == -1) ? 1 : 0;
		foreach my $param (@_) {
			if ($xx =~ /$param/is) {
				$ok = 1;
				last;
			}
		}
		($ok) || next;

		($_) = $xx =~ /($match)/is;
		s/\n//g;
		$ENV{'DEBUG'} && print "match: $_\n";
		&uniq_push(\@ret, &ComicGet::Web::complete_uri($_));
	}

	return @ret;
}


=item follow_rules
Follow rules to fetch target.

0: Rules
1: Target file
r: 1 on success.
=cut
sub follow_rules($) {
	my ($rules, $target) = @_;
	my ($data, $uri_id);
	my $title;
	my @uri_list;

	$file_no++;

	foreach my $line (split(/\n/, $rules)) {
		# Remove comments and trailing whitespaces
		$line =~ s/\\#/\n/;	# Temporarily convert escaped #s into lf
		$line =~ s/#.*//s;	# Strip comments
		$line =~ s/\s+$//s;	# Strip whitespaces
		$line =~ s/\n/#/s;	# Put #s back

		next if (length($line) == 0);
	
		my (@stuff) = split(/\s+/, $line);
		my $cmd = shift @stuff;
		my $opt = join(" ", @stuff);

		if ($cmd eq "title") {
			&ComicGet::Web::clear_referer();
			&ComicGet::Web::clear_cookies();
			$title = $opt;
			undef @uri_list;
		}
		
		elsif ($cmd eq "uri") {
			my ($host) = $stuff[0] =~ /^(.*?\/\/.*?\/)/;
			$act_line = "Get: $file_no $host $title";
			print "$act_line\r";
			flush STDOUT;

			push(@uri_list, $stuff[0]);
		}
		
		elsif ($cmd eq "fetch") {
			print "$act_line [fetch]\r";
			flush STDOUT;
			undef $uri_id;
			my $uri;
			my $raw_match;
			if ($#stuff == -1) {
				if ($#uri_list > 0) {
					print "\nE: Severals URIs match:\n";
					foreach (@uri_list) {
						print " - $_\n";
					}
					print "W: Skipping...\n";
					return 0;
				}
				
				$raw_match = $uri_list[0];
			} else {
				foreach my $id (@stuff) {
					foreach my $cand (@uri_list) {
						if ($cand =~ /^$id\t/) {
							$raw_match = $cand;
							last;
						}
					}
				}
			}
			if (! defined($raw_match)) {
				print "\nW: No appropriate URI found. " .
					"Skipping...\n";
				return 0;
			}
			my @t = split(/\t/, $raw_match, 2);
			if ($#t == 0) {
				$uri = $t[0];
			} else {
				$uri_id = $t[0];
				$uri = $t[1];
			}

			&ComicGet::Web::set_actline($act_line);
			$data = &ComicGet::Web::http_get($uri, 80);
			&ComicGet::Web::set_actline(undef);
			if ($ComicGet::Web::ret_code != $ComicGet::Web::OK) {
				print "\n";
				&ComicGet::Web::print_error();
				print "W: Skipping...\n";
				return 0;
			}
			my $size = &ComicGet::Common::human_readable(
					length($data));
			print "$act_line [$size]  \n";
			undef @uri_list;
		}
		
		elsif ($cmd eq "image") {
			my $type = "";
			$type = shift @stuff if ($#stuff > 0);
			my @t = &get($data, "image", @stuff);
			next if ($#t == -1);
			push(@uri_list, "$type\t$_") foreach (@t);
		}
		
		elsif ($cmd eq "imagelink") {
			my $type = "";
			$type = shift @stuff if ($#stuff > 0);
			my @t = &get($data, "link", @stuff);
			next if ($#t == -1);
			push(@uri_list, "$type\t$_") foreach (@t);
		}

		elsif ($cmd eq "select") {
			my $n = shift @stuff;
			if (! defined($n) || $n == 0) {
				print "\nE: Bad parameter for \"select\"!\n";
				print "W: Skipping...\n";
				return 0;
			}
			if ($n >= 1) {
				$n--;
			} else { # $n <= -1
				$n = $#uri_list + $n +1;
			}
			if (! defined($uri_list[$n])) {
				print "\nE: \"select\" out of range!\n";
				print "W: Skipping...\n";
				return 0;
			}
			@uri_list = ( $uri_list[$n] );
		}

		elsif ($cmd eq "link") {
			my @t = &get($data, "link", @stuff);
			next if ($#t == -1);
			push(@uri_list, @t);
		}
		
		elsif ($cmd eq "frame") {
			my @t = &get($data, "frame", @stuff);
			next if ($#t == -1);
			push(@uri_list, @t);
		}

		elsif ($cmd eq "tag") {
			my $type = $stuff[0];
			shift @stuff;

			my @t = &get_tag($data, @stuff);
			next if ($#t == -1);
			push(@uri_list, "$type\t$_") foreach (@t);
		}

		elsif ($cmd eq "save") {
			if (! defined($data)) {
				print "\nE: \"save\" has no data to save!\n";
				print "W: Skipping...\n";
				return 0;
			}

			&ComicGet::Config::read();
			my $path = $ComicGet::Config::target_dir;

			my $orig_file = shift @stuff;
			if (! defined($target)) {
				print "\nW: Database has target filename " .
					"missing. Using $orig_file instead.\n";
				$target = $orig_file;
			}
			my $file = "$path/$target";

			my $size = length($data);
			my $convert;
			foreach my $extra (@stuff) {
				if ($extra =~ /^convert:/) {
					my ($t) = $extra =~ /convert:(.*)/;
					foreach my $id (split(/,/, $t)) {
						if ($id eq $uri_id) {
							$convert = $id;
							last;
						}
					}
				}
			}

			$size = &ComicGet::Common::human_readable($size);

			if (defined($convert)) {
				print "$act_line [convert]  \r";
				flush STDOUT;
				open(PIPE, "| convert - $file")
						or die("Cannot convert: $!");
				print PIPE $data;
				close(PIPE);
			} else {
				&ComicGet::Common::save_file("$file", $data)
					or die("Cannot write \"$file\": " .
						&ComicGet::Common::error);
			}

			my ($trim) = $file =~ /.*\/(.*)$/;
			print "$act_line [$size] $trim  \n";
			flush STDOUT;
		}
		
		elsif ($cmd eq "stop") {
			die("Stopped");
		}

		elsif ($cmd eq "sort") {
			if (!defined($opt) || $opt eq "desc") {
				@uri_list = reverse sort sort_uri @uri_list;
			} else {
				@uri_list = sort sort_uri @uri_list;
			}
		}

		elsif ($cmd eq "--info--") {
			return 1;
		}
		
		else {
			die("Unrecognized command: $cmd");
		}
	}

	print "\nW: Rulefile ended unexpectedly.\n";
	return 0;
}



=sort_uri
URI sort helper, ignores type

0: A
1: B
=cut
sub sort_uri($$) {
	my ($a, $b) = @_;
	$a =~ s/.*\t//;
	$b =~ s/.*\t//;
	$a cmp $b;
}


=item get_file
Get filename from rules file.

0: Rule info
r: Filename
=cut
sub get_file($) {
	my $rules = shift;

	foreach my $line (split(/\n/, $rules)) {
		my @t = split(/\s+/, $line);
		(defined($t[0]) && $t[0] eq "save") && return $t[1];
	}
	return undef;
}


1;
