#! /usr/bin/perl
#
# comic-get.pl - Utility for fetching comics from the internet
# Copyright (C) 2004-2007 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# BUGS:
# - Cookie-support has some bugs. This implementation works with my comics.
#   If you can't live with it, you fix it. I'm having a day off now.


use strict;
use warnings FATAL => 'all';
use diagnostics;

use ComicGet::Web;
use ComicGet::Common;
use ComicGet::Config;
use ComicGet::Repository;

while ($#ARGV >= 0 && $ARGV[0] eq "-v") {
	$ComicGet::Config::verbose++;
	shift @ARGV;
}


my $cmd = shift @ARGV;
my $usage = 0;
my $verbose = 0;


sub check_home() {
	if (-e $ComicGet::Config::home) {
		if (! -d $ComicGet::Config::home) {
			print "E: \"$ComicGet::Config::home\" " .
				"is not a directory!\n";
			exit(1);
		}
	} else {
		print "E: \"$ComicGet::Config::home\" missing.\n";
		exit(0);
	}
}


if (! defined($cmd)) {
	$usage = 1;
} elsif ($cmd eq "search") {
	&check_home();
	&ComicGet::Repository::search($ComicGet::Repository::SEARCH, @ARGV);
} elsif ($cmd eq "show") {
	&check_home();
	&ComicGet::Repository::search($ComicGet::Repository::SHOW, @ARGV);
} elsif ($cmd eq "update") {
	&check_home();
	&ComicGet::Repository::update();
} elsif ($cmd eq "upgrade" || $cmd eq "refresh") {
	&check_home();
	&ComicGet::Repository::upgrade(@ARGV);
} elsif ($cmd eq "install") {
	&check_home();
	&ComicGet::Repository::install(@ARGV);
} elsif ($cmd eq "remove") {
	&check_home();
	&ComicGet::Repository::remove(@ARGV);
} elsif ($cmd eq "list") {
	&check_home();
	&ComicGet::Database::list($ARGV[0]);
} elsif ($cmd eq "pkgnames") {
	&check_home();
	&ComicGet::Repository::search($ComicGet::Repository::SEARCH, "");
} elsif ($cmd eq "masterpool") {
	&check_home();
	&ComicGet::Config::read();
	&ComicGet::Web::set_actline(undef);
	my $data = &ComicGet::Web::http_get($ComicGet::Config::repo_pool, 80);
	if ($ComicGet::Web::ret_code == $ComicGet::Web::OK) {
		print "Known repositories according to master " .
				"repository pool:\n";
		print "$data";
	} else {
		&ComicGet::Web::print_error();
		exit(1);
	}
} elsif ($cmd eq "test") {
	&check_home();
	my $file = $ARGV[0];
	&ComicGet::Config::read();
	my $data = &ComicGet::Common::read_file($file);
	if (! defined($data)) {
		print "E: Cannot read \"$file\": $ComicGet::Common::error\n";
		exit(1);
	}
	&ComicGet::Fetch::follow_rules($data);
} else {
	$usage = 1;
}

if ($usage == 1) {
	print <<EOF;
comic-get 0.3.3
Usage: comic-get command parameters ...

       comic-get update|upgrade|refresh
       comic-get install|remove pkg1 [pkg2 ...]
       comic-get search rule1 [rule2 ...]
       comic-get show pkg1 [pkg2 ...]
       comic-get list [regexp]
       comic-get pkgnames
       comic-get test file

Commands:
   update   - Retrieve new list of comics
   upgrade  - Perform an upgrade (refresh comics)
   install  - Install new comics
   remove   - Remove comics
   search   - Search the comic list for a pattern
   show     - Show a readable record for the comic strip
   list     - List installed packages (matching regexp)
   pkgnames - List the names of all packages
   test     - Try out a new rule
EOF
}


exit(0);
